#
# Copyright 2019 Bernhard Walter
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import base64

from traitlets import Unicode, List, Dict

from ipywidgets.widgets import register, Button
from ipywidgets.widgets.trait_types import bytes_serialization

UNSELECTED = 0
SELECTED = 1
MIXED = 2
EMPTY = 3


def state_diff(old_states, new_states):
    result = []
    for key in old_states.keys():
        if old_states[key] != new_states[key]:
            for i in range(len(old_states[key])):
                if old_states[key][i] != new_states[key][i]:
                    result.append({key: {"icon": i, "new": new_states[key][i]}})
    return result


@register
class TreeView(Button):
    """An example widget."""

    _view_name = Unicode("TreeView").tag(sync=True)
    _model_name = Unicode("TreeModel").tag(sync=True)
    _view_module = Unicode("jupyter_cadquery").tag(sync=True)
    _model_module = Unicode("jupyter_cadquery").tag(sync=True)
    _view_module_version = Unicode("2.0.2").tag(sync=True)
    _model_module_version = Unicode("2.0.2").tag(sync=True)
    image_paths = List(Dict(None, allow_none=True))
    icons = List(Dict(None, allow_none=True)).tag(sync=True)
    tree = Dict(None, allow_none=True).tag(sync=True)
    state = Dict(None, allow_none=True).tag(sync=True)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        icons = []
        for image_set in self.image_paths:
            icons.append({k: self._load_image(v) for k, v in image_set.items()})
        self.icons = icons

    def _load_image(self, image_path):
        if image_path == "":
            return b""
        else:
            return base64.b64encode(open(image_path, "rb").read()).decode()



"""
jupyter_cadquery_widgets setup
"""
import json
from pathlib import Path

from jupyter_packaging import (
    create_cmdclass,
    install_npm,
    ensure_targets,
    combine_commands,
    skip_if_exists,
)
import setuptools

HERE = Path(__file__).parent.resolve()

# The name of the project
name = "jupyter_cadquery_widgets"
py_path = HERE / name
lab_path = HERE / name / "labextension"
js_path = HERE / "lib"

# Representative files that should exist after a successful build
# jstargets = [
#     str(lab_path / "package.json"),
# ]

package_data_spec = {
    name: ["*"],
}

labext_name = "jupyter_cadquery"

data_files_spec = [
    # JupyterLab 3
    ("share/jupyter/labextensions/%s" % labext_name, str(lab_path), "**"),
    ("share/jupyter/labextensions/%s" % labext_name, str(HERE), "install.json"),
    # Jupyter Notebook
    ("etc/jupyter/nbconfig", str(HERE / "jupyter-config"), "**/*.json"),
    ("share/jupyter/nbextensions/jupyter_cadquery", str(py_path / "nbextension"), "**/*.js"),
    ("share/jupyter/nbextensions/jupyter_cadquery", str(py_path / "nbextension"), "**/*.js.map"),
]

cmdclass = create_cmdclass(
    "jsdeps", package_data_spec=package_data_spec, data_files_spec=data_files_spec
)

cmdclass["jsdeps"] = combine_commands(
    install_npm(
        path=str(js_path),
        npm=["yarn"],
        build_cmd="build:prod",
    ),
    ensure_targets(
        [
            str(py_path / "nbextension" / "extension.js"),
            str(py_path / "nbextension" / "index.js"),
        ]
    ),
)

long_description = (HERE / "README.md").read_text()

# Get the package info from package.json
pkg_json = json.loads((HERE / "package.json").read_bytes())

setup_args = dict(
    name=name,
    version=pkg_json["version"],
    url=pkg_json["homepage"],
    author=pkg_json["author"]["name"],
    author_email=pkg_json["author"]["email"],
    description=pkg_json["description"],
    license=pkg_json["license"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    cmdclass=cmdclass,
    packages=setuptools.find_packages(),
    install_requires=["jupyterlab~=3.0", "ipywidgets~=7.6"],
    extras_require={
        "dev": {
            "jupyter-packaging",
            "cookiecutter",
            "twine",
            "bumpversion",
            "black",
            "pylint",
            "pyYaml",
        }
    },
    zip_safe=False,
    include_package_data=True,
    python_requires=">=3.6",
    platforms="Linux, Mac OS X, Windows",
    keywords=["Jupyter", "JupyterLab", "JupyterLab3"],
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Framework :: Jupyter",
    ],
)


if __name__ == "__main__":
    setuptools.setup(**setup_args)



#
# Copyright 2019 Bernhard Walter
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from traitlets import Unicode, validate, CUnicode, Bytes

from ipywidgets.widgets import register, Button
from ipywidgets.widgets.trait_types import bytes_serialization


@register
class ImageButton(Button):
    """An example widget."""

    _view_name = Unicode("ImageButtonView").tag(sync=True)
    _model_name = Unicode("ImageButtonModel").tag(sync=True)
    _view_module = Unicode("jupyter_cadquery").tag(sync=True)
    _model_module = Unicode("jupyter_cadquery").tag(sync=True)
    _view_module_version = Unicode("2.0.2").tag(sync=True)
    _model_module_version = Unicode("2.0.2").tag(sync=True)
    image_path = Unicode("")
    value = Bytes().tag(sync=True, **bytes_serialization)
    width = CUnicode("36", help="Width of the image in pixels.").tag(sync=True)
    height = CUnicode("28", help="Height of the image in pixels.").tag(sync=True)
    type = Unicode("").tag(sync=True)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.value = self._load_image(self.image_path)

    def _load_image(self, image_path):
        if image_path == "":
            return b""
        else:
            return open(image_path, "rb").read()
